
# q-expansion of newform 150.4.c.c, downloaded from the LMFDB on 22 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 150
weight = 4
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[101, [1, 0]], [127, [-1, 0]]]
aps_data = [[0, 2], [0, -3], [0, 0], [0, -4], [-48, 0], [0, -2], [0, -114], [-140, 0], [0, -72], [-210, 0], [272, 0], [0, -334], [-198, 0], [0, 268], [0, 216], [0, 78], [-240, 0], [302, 0], [0, 596], [-768, 0], [0, 478], [640, 0], [0, 348], [-210, 0], [0, -1534], [1722, 0], [0, -1052], [0, -564], [610, 0], [0, -1302], [0, -124], [192, 0], [0, -2514], [-1340, 0], [-1410, 0], [-2128, 0], [0, 3026], [0, -2612], [0, -24], [0, -1962], [120, 0], [902, 0], [-168, 0], [0, 1318], [0, -4014], [-2000, 0], [-3868, 0], [0, 3148], [0, 2556], [610, 0], [0, 2058], [-4920, 0], [-1438, 0], [792, 0], [0, 2166], [0, -3192], [-5490, 0], [-6328, 0], [0, -574], [4242, 0], [0, 628], [0, 558], [0, -6964], [2832, 0], [0, -8642], [0, -2214], [10772, 0], [0, -1654], [0, 2196], [-8270, 0], [0, -10302], [2280, 0], [0, -8764], [0, 1318], [-1100, 0], [0, 3528], [9630, 0], [0, -3094], [-1638, 0], [13750, 0], [12480, 0], [7262, 0], [9792, 0], [0, -1802], [2320, 0], [0, -11172], [-6810, 0], [0, 17066], [-18918, 0], [0, -1052], [0, 11076], [9000, 0], [0, -8764], [5592, 0], [-4700, 0], [0, 11808], [-1170, 0], [-16638, 0], [0, -15692], [-22018, 0], [0, -4564], [0, -7734], [0, 20148], [24030, 0], [2372, 0], [0, 8546], [0, -15444], [0, -18342], [-24600, 0], [-8998, 0], [0, 4076], [0, 4078], [0, 10086], [-8780, 0], [2792, 0], [7602, 0], [0, -24212], [0, 9456], [0, 9558], [29280, 0], [-29098, 0], [0, 11638], [0, 3426], [0, 20148], [-29428, 0], [16242, 0], [-2030, 0], [-6960, 0], [0, 18596], [0, -21242], [340, 0], [0, 21888], [17792, 0], [0, 37346], [-11358, 0], [34270, 0], [0, 13278], [0, -11164], [0, -5094], [8790, 0], [5852, 0], [-29478, 0], [0, -39332], [0, 6756], [-3950, 0], [-12360, 0], [0, 35998], [0, -21594], [-9260, 0], [0, -31632], [0, -39694], [1242, 0], [0, 2668], [0, -4344], [0, 4436], [22752, 0], [27160, 0], [33030, 0], [0, -29974], [13962, 0], [0, 35196], [0, 28338], [0, -17524], [-26808, 0], [0, -10914], [0, -22272], [14072, 0], [0, 4826]]
